import logging

from ruxit.api.base_plugin import RemoteBasePlugin

from zabbix_api import Zabbix

logger = logging.getLogger(__name__)

severities = {
    "0": "NOT CLASSIFIED",
    "1": "INFORMATION",
    "2": "WARNING",
    "3": "AVERAGE",
    "4": "HIGH",
    "5": "DISASTER",
}


class ZabbixPlugin(RemoteBasePlugin):
    def query(self, **kwargs) -> None:

        config = kwargs["config"]
        severities_filter = config.get("zabbix_severities", "").split(",")
        group = self.topology_builder.create_group("Zabbix", "Zabbix")
        hosts = {}

        zabbix_groups = config.get("zabbix_groups", "").split(",")

        z = Zabbix(config["zabbix_url"], config["zabbix_username"], config["zabbix_password"], logger=logger,)
        try:
            for host in z.get_hosts(zabbix_groups):
                groups = ",".join([group["name"] for group in host["groups"]])
                device = group.create_device(f'{host["hostid"]}', f'{host["name"]} ({groups})')
                device.report_property("Groups", groups)
                device.report_property("Host ID", host["hostid"])
                for interface in host["interfaces"]:
                    device.add_endpoint(interface["ip"])

                device.absolute("monitored", 100 * int(host["status"] == "0"))
                hosts[host["hostid"]] = device

            problems = z.get_problems()
            events = z.get_events(event_ids=[problem["eventid"] for problem in problems])

            for event in events:
                if not severities_filter or event["severity"] in severities_filter:
                    for host in event["hosts"]:
                        try:
                            # We get all events, but not all hosts, so the host might not be available at this time
                            device = hosts.get(host["hostid"], None)
                            if device is not None:
                                properties = {
                                    "Event ID": event["eventid"],
                                    "Trigger ID": event["objectid"],
                                    "Event URL": f'{config["zabbix_url"].replace("api_jsonrpc.php", "tr_events.php")}?triggerid={event["objectid"]}&eventid={event["eventid"]}',
                                }
                                description = f'{event["name"]}' "ty"
                                title = f"Zabbix - {severities[event['severity']]} Event {event['eventid']}"
                                device.report_error_event(description, title, properties=properties)
                        except Exception as e:
                            logger.exception(f"Error sending event: {e}")
        except Exception as e:
            logger.exception(f"Error executing Zabbix: {e}")
